from _typeshed import Incomplete
from talon import app as app, storage as storage, ui as ui
from talon.api import ffi as ffi, ffi_string as ffi_string, lib as lib
from talon.scripting import Dispatch as Dispatch, rctx as rctx
from talon.skia import Canvas as SkCanvas
from talon.types import Point2d as Point2d, Rect as Rect
from typing import Any, Generator, Iterable, Optional

class MouseEvent:
    event: str
    button: Optional[int]
    clicks: Optional[int]
    mods: frozenset[str]
    pos: Point2d
    gpos: Point2d
    down: bool
    up: bool
    move: bool
    def __init__(self, event, button, clicks, mods, pos, gpos, down, up, move) -> None: ...

class KeyEvent:
    event: str
    code: int
    key: Optional[str]
    mods: frozenset[str]
    repeat: bool
    down: bool
    up: bool
    def __init__(self, event, code, key, mods, repeat, down, up) -> None: ...

class TextEvent:
    event: str
    text: str
    def __init__(self, event, text) -> None: ...

class ScrollEvent:
    event: str
    mods: frozenset[str]
    pos: Point2d
    gpos: Point2d
    pixels: Optional[Point2d]
    degrees: Optional[Point2d]
    def __init__(self, event, mods, pos, gpos, pixels, degrees) -> None: ...

class Canvas(Dispatch):
    @classmethod
    def from_screen(cls, screen: ui.Screen, **kwargs: Any) -> Canvas: ...
    @classmethod
    def from_rect(cls, rect: Rect, **kwargs: Any) -> Canvas: ...
    actor: Incomplete
    ctx: Incomplete
    name: Incomplete
    paused: Incomplete
    def __init__(self, x: float, y: float, width: float, height: float, *, paused: bool = ..., name: str = ..., backend: str = ..., software: bool = ..., panel: bool = ..., block_mouse: bool = ..., fullscreen: bool = ..., draggable: bool = ...) -> None: ...
    def on_draw(self, handle: int, x: int, y: int, width: int, height: int) -> None: ...
    def on_mouse(self, event) -> None: ...
    def on_click(self, event) -> None: ...
    def on_scroll(self, event) -> None: ...
    def on_key(self, event) -> None: ...
    def on_text(self, ch: str) -> None: ...
    def on_mods(self, event) -> None: ...
    def on_moved(self, x: int, y: int) -> None: ...
    def on_resized(self, width: int, height: int) -> None: ...
    def on_focused(self, focused: bool) -> None: ...
    def pause(self) -> None: ...
    def resume(self) -> None: ...
    def hide(self) -> None: ...
    def show(self) -> None: ...
    def freeze(self) -> None: ...
    def register_first(self, topic: str) -> None: ...
    def unregister_last(self, topic: str) -> None: ...
    @property
    def rect(self) -> Rect: ...
    def move(self, x: float, y: float) -> None: ...
    def resize(self, width: float, height: float) -> None: ...
    def resize_from_screen(self, screen: ui.Screen) -> None: ...
    @property
    def panel(self) -> bool: ...
    @property
    def fullscreen(self) -> bool: ...
    @property
    def draggable(self) -> bool: ...
    @property
    def allows_capture(self) -> bool: ...
    def disable_capture(self) -> Generator[None, None, None]: ...
    @property
    def cursor_visible(self) -> bool: ...
    @property
    def blocks_mouse(self) -> bool: ...
    @property
    def focused(self) -> bool: ...
    @property
    def title(self) -> str: ...
    def close(self) -> None: ...

class Overlay(Dispatch):
    canvases: list[Canvas]
    def __init__(self) -> None: ...
    def draw(self, c: SkCanvas) -> bool: ...
    @property
    def allows_capture(self) -> bool: ...
    def register_first(self, topic: str) -> None: ...
    def unregister_last(self, topic: str) -> None: ...
    def recreate(self, screens: Iterable[ui.Screen], force: bool = ...): ...
    def screen_update(self, screens: Iterable[ui.Screen]) -> None: ...

overlay: Incomplete
register: Incomplete
unregister: Incomplete
